@@ -42,7 +42,7 @@ class DialogDisplay:
         ('focus','white','dark blue','bold'),
         ('focustext','light gray','dark blue'),
         ]
-        
+
     def __init__(self, text, height, width, body=None):
         width = int(width)
         if width <= 0:
@@ -50,7 +50,7 @@ class DialogDisplay:
         height = int(height)
         if height <= 0:
             height = ('relative', 80)
-    
+
         self.body = body
         if body is None:
             # fill space with nothing
@@ -61,24 +61,24 @@ class DialogDisplay:
             self.frame.header = urwid.Pile( [urwid.Text(text),
                 urwid.Divider()] )
         w = self.frame
-        
+
         # pad area around listbox
         w = urwid.Padding(w, ('fixed left',2), ('fixed right',2))
         w = urwid.Filler(w, ('fixed top',1), ('fixed bottom',1))
         w = urwid.AttrWrap(w, 'body')
-        
+
         # "shadow" effect
         w = urwid.Columns( [w,('fixed', 2, urwid.AttrWrap(
             urwid.Filler(urwid.Text(('border','  ')), "top")
             ,'shadow'))])
-        w = urwid.Frame( w, footer = 
+        w = urwid.Frame( w, footer =
             urwid.AttrWrap(urwid.Text(('border','  ')),'shadow'))
 
         # outermost border area
         w = urwid.Padding(w, 'center', width )
         w = urwid.Filler(w, 'middle', height )
         w = urwid.AttrWrap( w, 'border' )
-        
+
         self.view = w
 
 
@@ -102,10 +102,10 @@ class DialogDisplay:
             self.loop.run()
         except DialogExit, e:
             return self.on_exit( e.args[0] )
-        
+
     def on_exit(self, exitcode):
         return exitcode, ""
-        
+
 
 
 class InputDialogDisplay(DialogDisplay):
@@ -113,11 +113,11 @@ class InputDialogDisplay(DialogDisplay):
         self.edit = urwid.Edit()
         body = urwid.ListBox([self.edit])
         body = urwid.AttrWrap(body, 'selectable','focustext')
-        
+
         DialogDisplay.__init__(self, text, height, width, body)
-        
+
         self.frame.set_focus('body')
-    
+
     def unhandled_key(self, size, k):
         if k in ('up','page up'):
             self.frame.set_focus('body')
@@ -127,11 +127,11 @@ class InputDialogDisplay(DialogDisplay):
             # pass enter to the "ok" button
             self.frame.set_focus('footer')
             self.view.keypress( size, k )
-    
+
     def on_exit(self, exitcode):
         return exitcode, self.edit.get_edit_text()
 
-    
+
 class TextDialogDisplay(DialogDisplay):
     def __init__(self, file, height, width):
         l = []
@@ -154,20 +154,20 @@ class TextDialogDisplay(DialogDisplay):
 class ListDialogDisplay(DialogDisplay):
     def __init__(self, text, height, width, constr, items, has_default):
         j = []
-        if has_default:    
+        if has_default:
             k, tail = 3, ()
-        else:    
+        else:
             k, tail = 2, ("no",)
         while items:
             j.append( items[:k] + tail )
             items = items[k:]
-                    
+
         l = []
         self.items = []
         for tag, item, default in j:
             w = constr( tag, default=="on" )
             self.items.append(w)
-            w = urwid.Columns( [('fixed', 12, w), 
+            w = urwid.Columns( [('fixed', 12, w),
                 urwid.Text(item)], 2 )
             w = urwid.AttrWrap(w, 'selectable','focus')
             l.append(w)
@@ -175,9 +175,9 @@ class ListDialogDisplay(DialogDisplay):
         lb = urwid.ListBox(l)
         lb = urwid.AttrWrap( lb, "selectable" )
         DialogDisplay.__init__(self, text, height, width, lb )
-        
+
         self.frame.set_focus('body')
-    
+
     def unhandled_key(self, size, k):
         if k in ('up','page up'):
             self.frame.set_focus('body')
@@ -199,14 +199,14 @@ class ListDialogDisplay(DialogDisplay):
                 s = i.get_label()
                 break
         return exitcode, s
-        
-    
-    
+
+
+
 
 class CheckListDialogDisplay(ListDialogDisplay):
     def on_exit(self, exitcode):
         """
-        Mimic dialog(1)'s --checklist exit. 
+        Mimic dialog(1)'s --checklist exit.
         Put each checked item in double quotes with a trailing space.
         """
         if exitcode != 0:
@@ -250,7 +250,7 @@ def do_checklist(text, height, width, list_height, *items):
     d = CheckListDialogDisplay( text, height, width, constr, items, True)
     d.add_buttons([    ("OK", 0), ("Cancel", 1) ])
     return d
-    
+
 def do_inputbox(text, height, width):
     d = InputDialogDisplay( text, height, width )
     d.add_buttons([    ("Exit", 0) ])
@@ -286,22 +286,22 @@ def do_yesno(text, height, width):
     d.add_buttons([    ("Yes", 0), ("No", 1) ])
     return d
 
-MODES={    '--checklist':    (do_checklist, 
+MODES={    '--checklist':    (do_checklist,
         "text height width list-height [ tag item status ] ..."),
-    '--inputbox':    (do_inputbox, 
+    '--inputbox':    (do_inputbox,
         "text height width"),
-    '--menu':    (do_menu, 
+    '--menu':    (do_menu,
         "text height width menu-height [ tag item ] ..."),
-    '--msgbox':    (do_msgbox, 
+    '--msgbox':    (do_msgbox,
         "text height width"),
-    '--radiolist':    (do_radiolist, 
+    '--radiolist':    (do_radiolist,
         "text height width list-height [ tag item status ] ..."),
     '--textbox':    (do_textbox,
         "file height width"),
-    '--yesno':    (do_yesno, 
+    '--yesno':    (do_yesno,
         "text height width"),
     }
-    
+
 
 def show_usage():
     """
@@ -310,7 +310,7 @@ def show_usage():
     modelist = [(mode, help) for (mode, (fn, help)) in MODES.items()]
     modelist.sort()
     sys.stdout.write(
-        __doc__ + 
+        __doc__ +
         "\n".join(["%-15s %s"%(mode,help) for (mode,help) in modelist])
         + """
 
@@ -321,23 +321,23 @@ status may be either on or off.
 
 
 def main():
-    if len(sys.argv) < 2 or not MODES.has_key(sys.argv[1]):
+    if len(sys.argv) < 2 or sys.argv[1] not in MODES:
         show_usage()
         return
-    
+
     # Create a DialogDisplay instance
     fn, help = MODES[sys.argv[1]]
     d = fn( * sys.argv[2:] )
-    
+
     # Run it
     exitcode, exitstring = d.main()
-    
+
     # Exit
     if exitstring:
         sys.stderr.write(exitstring+"\n")
-    
-    sys.exit(exitcode)
-        
 
-if __name__=="__main__": 
+    sys.exit(exitcode)
+
+
+if __name__=="__main__":
     main()
