@@ -102,12 +102,10 @@ class Screen(BaseScreen, RealTerminal):
 
         self._mouse_tracking_enabled = enable
 
-    def start(self):
+    def _start(self):
         """
         Initialize the screen and input mode.
         """
-        assert self._started == False
-
         self.s = curses.initscr()
         self.has_color = curses.has_colors()
         if self.has_color:
@@ -126,44 +124,28 @@ class Screen(BaseScreen, RealTerminal):
         curses.meta(1)
         curses.halfdelay(10) # use set_input_timeouts to adjust
         self.s.keypad(0)
-        
+
         if not self._signal_keys_set:
             self._old_signal_keys = self.tty_signal_keys()
 
-        super(Screen, self).start()
+        super(Screen, self)._start()
 
-    
-    def stop(self):
+    def _stop(self):
         """
         Restore the screen.
         """
-        if self._started == False:
-            return
         curses.echo()
         self._curs_set(1)
         try:
             curses.endwin()
         except _curses.error:
             pass # don't block original error with curses error
-        
+
         if self._old_signal_keys:
             self.tty_signal_keys(*self._old_signal_keys)
 
-        super(Screen, self).stop()
+        super(Screen, self)._stop()
 
-    
-    def run_wrapper(self,fn):
-        """Call fn in fullscreen mode.  Return to normal on exit.
-        
-        This function should be called to wrap your main program loop.
-        Exception tracebacks will be displayed in normal mode.
-        """
-    
-        try:
-            self.start()
-            return fn()
-        finally:
-            self.stop()
 
     def _setup_colour_pairs(self):
         """
@@ -185,7 +167,7 @@ class Screen(BaseScreen, RealTerminal):
                 curses.init_pair(bg * 8 + 7 - fg, fg, bg)
 
     def _curs_set(self,x):
-        if self.cursor_state== "fixed" or x == self.cursor_state: 
+        if self.cursor_state== "fixed" or x == self.cursor_state:
             return
         try:
             curses.curs_set(x)
@@ -193,12 +175,12 @@ class Screen(BaseScreen, RealTerminal):
         except _curses.error:
             self.cursor_state = "fixed"
 
-    
+
     def _clear(self):
         self.s.clear()
         self.s.refresh()
-    
-    
+
+
     def _getch(self, wait_tenths):
         if wait_tenths==0:
             return self._getch_nodelay()
@@ -208,7 +190,7 @@ class Screen(BaseScreen, RealTerminal):
             curses.halfdelay(wait_tenths)
         self.s.nodelay(0)
         return self.s.getch()
-    
+
     def _getch_nodelay(self):
         self.s.nodelay(1)
         while 1:
@@ -218,17 +200,17 @@ class Screen(BaseScreen, RealTerminal):
                 break
             except _curses.error:
                 pass
-            
+
         return self.s.getch()
 
-    def set_input_timeouts(self, max_wait=None, complete_wait=0.1, 
+    def set_input_timeouts(self, max_wait=None, complete_wait=0.1,
         resize_wait=0.1):
         """
         Set the get_input timeout values.  All values have a granularity
         of 0.1s, ie. any value between 0.15 and 0.05 will be treated as
         0.1 and any value less than 0.05 will be treated as 0.  The
         maximum timeout value for this module is 25.5 seconds.
-    
+
         max_wait -- amount of time in seconds to wait for input when
             there is no input pending, wait forever if None
         complete_wait -- amount of time in seconds to wait when
@@ -295,14 +277,14 @@ class Screen(BaseScreen, RealTerminal):
         keys, raw = self._get_input( self.max_tenths )
 
         # Avoid pegging CPU at 100% when slowly resizing, and work
-        # around a bug with some braindead curses implementations that 
-        # return "no key" between "window resize" commands 
+        # around a bug with some braindead curses implementations that
+        # return "no key" between "window resize" commands
         if keys==['window resize'] and self.prev_input_resize:
             while True:
                 keys, raw2 = self._get_input(self.resize_tenths)
                 raw += raw2
                 if not keys:
-                    keys, raw2 = self._get_input( 
+                    keys, raw2 = self._get_input(
                         self.resize_tenths)
                     raw += raw2
                 if keys!=['window resize']:
@@ -310,33 +292,33 @@ class Screen(BaseScreen, RealTerminal):
             if keys[-1:]!=['window resize']:
                 keys.append('window resize')
 
-                
+
         if keys==['window resize']:
             self.prev_input_resize = 2
         elif self.prev_input_resize == 2 and not keys:
             self.prev_input_resize = 1
         else:
             self.prev_input_resize = 0
-        
+
         if raw_keys:
             return keys, raw
         return keys
-        
-        
+
+
     def _get_input(self, wait_tenths):
-        # this works around a strange curses bug with window resizing 
+        # this works around a strange curses bug with window resizing
         # not being reported correctly with repeated calls to this
         # function without a doupdate call in between
-        curses.doupdate() 
-        
+        curses.doupdate()
+
         key = self._getch(wait_tenths)
         resize = False
         raw = []
         keys = []
-        
+
         while key >= 0:
             raw.append(key)
-            if key==KEY_RESIZE: 
+            if key==KEY_RESIZE:
                 resize = True
             elif key==KEY_MOUSE:
                 keys += self._encode_mouse_event()
@@ -345,7 +327,7 @@ class Screen(BaseScreen, RealTerminal):
             key = self._getch_nodelay()
 
         processed = []
-        
+
         try:
             while keys:
                 run, keys = escape.process_keyqueue(keys, True)
@@ -354,7 +336,7 @@ class Screen(BaseScreen, RealTerminal):
             key = self._getch(self.complete_tenths)
             while key >= 0:
                 raw.append(key)
-                if key==KEY_RESIZE: 
+                if key==KEY_RESIZE:
                     resize = True
                 elif key==KEY_MOUSE:
                     keys += self._encode_mouse_event()
@@ -369,23 +351,23 @@ class Screen(BaseScreen, RealTerminal):
             processed.append('window resize')
 
         return processed, raw
-        
-        
+
+
     def _encode_mouse_event(self):
         # convert to escape sequence
         last = next = self.last_bstate
         (id,x,y,z,bstate) = curses.getmouse()
-        
+
         mod = 0
         if bstate & curses.BUTTON_SHIFT:    mod |= 4
         if bstate & curses.BUTTON_ALT:        mod |= 8
         if bstate & curses.BUTTON_CTRL:        mod |= 16
-        
+
         l = []
         def append_button( b ):
             b |= mod
             l.extend([ 27, ord('['), ord('M'), b+32, x+33, y+33 ])
-        
+
         if bstate & curses.BUTTON1_PRESSED and last & 1 == 0:
             append_button( 0 )
             next |= 1
@@ -410,7 +392,7 @@ class Screen(BaseScreen, RealTerminal):
         if bstate & curses.BUTTON4_RELEASED and last & 8:
             append_button( 64 + escape.MOUSE_RELEASE_FLAG )
             next &= ~ 8
-        
+
         if bstate & curses.BUTTON1_DOUBLE_CLICKED:
             append_button( 0 + escape.MOUSE_MULTIPLE_CLICK_FLAG )
         if bstate & curses.BUTTON2_DOUBLE_CLICKED:
@@ -431,7 +413,7 @@ class Screen(BaseScreen, RealTerminal):
 
         self.last_bstate = next
         return l
-            
+
 
     def _dbg_instr(self): # messy input string (intended for debugging)
         curses.echo()
@@ -440,17 +422,17 @@ class Screen(BaseScreen, RealTerminal):
         str = self.s.getstr()
         curses.noecho()
         return str
-        
+
     def _dbg_out(self,str): # messy output function (intended for debugging)
         self.s.clrtoeol()
         self.s.addstr(str)
         self.s.refresh()
         self._curs_set(1)
-        
+
     def _dbg_query(self,question): # messy query (intended for debugging)
         self._dbg_out(question)
         return self._dbg_instr()
-    
+
     def _dbg_refresh(self):
         self.s.refresh()
 
@@ -460,7 +442,7 @@ class Screen(BaseScreen, RealTerminal):
         """Return the terminal dimensions (num columns, num rows)."""
         rows,cols = self.s.getmaxyx()
         return cols,rows
-        
+
 
     def _setattr(self, a):
         if a is None:
@@ -502,19 +484,19 @@ class Screen(BaseScreen, RealTerminal):
     def draw_screen(self, (cols, rows), r ):
         """Paint screen with rendered canvas."""
         assert self._started
-        
+
         assert r.rows() == rows, "canvas size and passed size don't match"
-    
+
         y = -1
         for row in r.content():
             y += 1
             try:
                 self.s.move( y, 0 )
             except _curses.error:
-                # terminal shrunk? 
+                # terminal shrunk?
                 # move failed so stop rendering.
                 return
-            
+
             first = True
             lasta = None
             nr = 0
@@ -558,7 +540,7 @@ class Screen(BaseScreen, RealTerminal):
         else:
             self._curs_set(0)
             self.s.move(0,0)
-        
+
         self.s.refresh()
         self.keep_cache_alive_link = r
 
@@ -585,7 +567,7 @@ class _test:
                 (c+" on light gray",c,'light gray', 'standout'),
                 ])
         self.ui.run_wrapper(self.run)
-        
+
     def run(self):
         class FakeRender: pass
         r = FakeRender()
@@ -593,12 +575,12 @@ class _test:
         attr = [[],[]]
         r.coords = {}
         r.cursor = None
-        
+
         for c in self.l:
             t = ""
             a = []
             for p in c+" on black",c+" on dark blue",c+" on light gray":
-                
+
                 a.append((p,27))
                 t=t+ (p+27*" ")[:27]
             text.append( t )
@@ -624,13 +606,13 @@ class _test:
                 t += "'"+k + "' "
                 a += [(None,1), ('yellow on dark blue',len(k)),
                     (None,2)]
-            
+
             text.append(t + ": "+ repr(raw))
             attr.append(a)
             text = text[-rows:]
             attr = attr[-rows:]
-                
-                
+
+
 
 
 if '__main__'==__name__:
