@@ -304,6 +304,7 @@ class BarGraph(Widget):
         if hlines is not None:
             hlines = hlines[:]  # shallow copy
             hlines.sort()
+            hlines.reverse()
         self.data = bardata, top, hlines
         self._invalidate()
 
@@ -802,7 +803,7 @@ class ProgressBar(Widget):
 
     def __init__(self, normal, complete, current=0, done=100, satt=None):
         """
-        :param normal: display attribute for uncomplete part of progress bar
+        :param normal: display attribute for incomplete part of progress bar
         :param complete: display attribute for complete part of progress bar
         :param current: current progress
         :param done: progress amount at 100%
